@@ -1,6 +1,8 @@
 package dotnet
 
 import (
+	"os"
+	"path/filepath"
 	"testing"
 
 	"github.com/stretchr/testify/assert"
@@ -9,6 +11,7 @@ import (
 	"github.com/pulumi/pulumi/pkg/v3/codegen/internal/test"
 	"github.com/pulumi/pulumi/pkg/v3/codegen/schema"
 	"github.com/pulumi/pulumi/pkg/v3/testing/integration"
+	"github.com/pulumi/pulumi/sdk/v3/go/common/util/contract"
 	"github.com/pulumi/pulumi/sdk/v3/go/common/util/executable"
 )
 
@@ -28,6 +31,10 @@ func typeCheckGeneratedPackage(t *testing.T, pwd string) {
 	dotnet, err = executable.FindExecutable("dotnet")
 	require.NoError(t, err)
 	cmdOptions := integration.ProgramTestOptions{}
+	versionPath := filepath.Join(pwd, "version.txt")
+	err = os.WriteFile(versionPath, []byte("0.0.0\n"), 0600)
+	defer func() { contract.IgnoreError(os.Remove(versionPath)) }()
+	require.NoError(t, err)
 	err = integration.RunCommand(t, "dotnet build", []string{dotnet, "build"}, pwd, &cmdOptions)
 	require.NoError(t, err)
 }
