@@ -939,7 +939,7 @@ func TestOutputValueMarshaling(t *testing.T) {
 			opts: MarshalOptions{KeepUnknowns: true},
 			raw:  resource.NewOutputProperty(resource.Output{}),
 			expected: &structpb.Value{
-				Kind: &structpb.Value_NullValue{NullValue: 0},
+				Kind: &structpb.Value_StringValue{StringValue: UnknownStringValue},
 			},
 		},
 		{
@@ -1650,3 +1650,122 @@ func walkValueSelfWithDescendants(
 	}
 	return nil
 }
+
+func TestMarshalPropertiesDontSkipOutputs(t *testing.T) {
+	tests := []struct {
+		name     string
+		opts     MarshalOptions
+		props    resource.PropertyMap
+		expected *structpb.Struct
+	}{
+		{
+			name: "Computed (KeepUnknowns)",
+			opts: MarshalOptions{KeepUnknowns: true},
+			props: resource.PropertyMap{
+				"message": resource.MakeComputed(resource.NewStringProperty("")),
+				"nested": resource.NewObjectProperty(resource.PropertyMap{
+					"value": resource.MakeComputed(resource.NewStringProperty("")),
+				}),
+			},
+			expected: &structpb.Struct{
+				Fields: map[string]*structpb.Value{
+					"message": {
+						Kind: &structpb.Value_StringValue{StringValue: UnknownStringValue},
+					},
+					"nested": {
+						Kind: &structpb.Value_StructValue{
+							StructValue: &structpb.Struct{
+								Fields: map[string]*structpb.Value{
+									"value": {
+										Kind: &structpb.Value_StringValue{StringValue: UnknownStringValue},
+									},
+								},
+							},
+						},
+					},
+				},
+			},
+		},
+		{
+			name: "Output (KeepUnknowns)",
+			opts: MarshalOptions{KeepUnknowns: true},
+			props: resource.PropertyMap{
+				"message": resource.NewOutputProperty(resource.Output{}),
+				"nested": resource.NewObjectProperty(resource.PropertyMap{
+					"value": resource.NewOutputProperty(resource.Output{}),
+				}),
+			},
+			expected: &structpb.Struct{
+				Fields: map[string]*structpb.Value{
+					"message": {
+						Kind: &structpb.Value_StringValue{StringValue: UnknownStringValue},
+					},
+					"nested": {
+						Kind: &structpb.Value_StructValue{
+							StructValue: &structpb.Struct{
+								Fields: map[string]*structpb.Value{
+									"value": {
+										Kind: &structpb.Value_StringValue{StringValue: UnknownStringValue},
+									},
+								},
+							},
+						},
+					},
+				},
+			},
+		},
+		{
+			name: "Output (KeepUnknowns, KeepOutputValues)",
+			opts: MarshalOptions{KeepUnknowns: true, KeepOutputValues: true},
+			props: resource.PropertyMap{
+				"message": resource.NewOutputProperty(resource.Output{}),
+				"nested": resource.NewObjectProperty(resource.PropertyMap{
+					"value": resource.NewOutputProperty(resource.Output{}),
+				}),
+			},
+			expected: &structpb.Struct{
+				Fields: map[string]*structpb.Value{
+					"message": {
+						Kind: &structpb.Value_StructValue{
+							StructValue: &structpb.Struct{
+								Fields: map[string]*structpb.Value{
+									resource.SigKey: {
+										Kind: &structpb.Value_StringValue{StringValue: resource.OutputValueSig},
+									},
+								},
+							},
+						},
+					},
+					"nested": {
+						Kind: &structpb.Value_StructValue{
+							StructValue: &structpb.Struct{
+								Fields: map[string]*structpb.Value{
+									"value": {
+										Kind: &structpb.Value_StructValue{
+											StructValue: &structpb.Struct{
+												Fields: map[string]*structpb.Value{
+													resource.SigKey: {
+														Kind: &structpb.Value_StringValue{
+															StringValue: resource.OutputValueSig,
+														},
+													},
+												},
+											},
+										},
+									},
+								},
+							},
+						},
+					},
+				},
+			},
+		},
+	}
+	for _, tt := range tests {
+		t.Run(tt.name, func(t *testing.T) {
+			actual, err := MarshalProperties(tt.props, tt.opts)
+			assert.NoError(t, err)
+			assert.Equal(t, tt.expected, actual)
+		})
+	}
+}
